/*
  centermarker.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "centermarker.h"
#include "layeredbase.h"
#include "mainwindow.h"
#include <qpixmap.h>

CenterMarker::CenterMarker(LayeredBase *parent, const char *name) :
	LayeredWidget(parent, name)
{
	QGConf *conf = (QGConf *)lookupObject(parent, "QGConf", "config");
	if(conf)
		marker_on = conf->readNumEntry("CenterMarker", true);
}

bool CenterMarker::paintEvent(QPaintEvent *e UNUSED, QPixmap &scr)
{
	if(!marker_on)
		return false;

	QRect r = geometry();
	QPoint c = r.center();

	QPainter p(&scr);
	p.setPen(Qt::black);
	p.drawLine(r.left(), c.y(), r.right(), c.y());
	p.drawLine(c.x(), r.top(), c.x(), r.bottom());

	return true;
}

void CenterMarker::toggleMarker(void)
{
	QGConf *conf = (QGConf *)lookupObject(this, "QGConf", "config");

	marker_on = !marker_on;
	if(conf)
		conf->writeEntry("CenterMarker", marker_on);
	repaint();
}

bool CenterMarker::keyPressEvent(QKeyEvent *e)
{
	if(e->key() == Key_Period && !(e->state() & ShiftButton)){
		toggleMarker();
		return true;
	}
	return false;
}

void CenterMarker::showEvent()
{
	int x = parentWidget()->width()/2;
	int y = parentWidget()->height()/2;

	setGeometry(x - CENTER_MARKER_SIZE, y - CENTER_MARKER_SIZE,
				CENTER_MARKER_SIZE * 2 + 1, CENTER_MARKER_SIZE * 2 + 1);
}

void CenterMarker::request(QString const &req, QString const &arg)
{
	if (req != "menu")
		return;

	if (arg.toInt() == EV_TOGGLE_CENTER_MARKER)
		toggleMarker();
}
