/*
  map_item.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG
#include "qgmap.h"
#include <qpixmap.h>
#include <qobject.h>
#include <qpainter.h>
#include "map_item.h"
#include <stdio.h>

void MapItem::drawImage(QImage &scr, const QImage *image, int dst_x, int dst_y)
{
	Enter();

	if (scr.depth() != 32) {
		scr = scr.convertDepth(32);
	}

	if (scr.depth() != 32 || image->depth() != 32) {
		printf("XXX Fatal: screen or image depth is not 32 bpp.\n");
		return;
	}

	if (scr.width() != IMAGE_SIZE || scr.height() != IMAGE_SIZE) {
		printf("XXX Fatal: screen size is not 256x256.\n");
		return;
	}

	int x, y;

	int iw = image->width();
	int ih = image->height();

	int sx_s = MIN(MAX(-dst_x, 0), iw);
	int sy_s = MIN(MAX(-dst_y, 0), ih);

	int sx_e = MIN(IMAGE_SIZE - dst_x, iw);
	int sy_e = MIN(IMAGE_SIZE - dst_y, ih);

	for (y = sy_s; y < sy_e; y++) {
		unsigned int *dst, *src;
		int dy = dst_y + y;
		src = (unsigned int *)image->scanLine(y);
		dst = (unsigned int *)scr.scanLine(dy);
		for (x = sx_s; x < sx_e; x++) {
			if (src[x] & 0x80000000)
				dst[dst_x + x] = src[x];
		}
	}
}
