/*
  poi_dialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "poi_dialog.h"
#include <qstring.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qsizepolicy.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include "poi_add_dialog.h"
#include "poi_db.h"
#include "progressdialog.h"
#include <stdio.h>
#include <qapplication.h>
#include <unistd.h>

#define DIALOG_WIDTH  400
#define DIALOG_HEIGHT 400

PoiDialog::PoiDialog( QWidget* parent, const char* name,
								bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
    resize( QSize(DIALOG_WIDTH, DIALOG_HEIGHT).expandedTo(minimumSizeHint()) );

	setCaption(QString::fromUtf8("カスタムPOI"));

//	setFont(QFont("lcfont", 16));

	QHBoxLayout *base_hbox = new QHBoxLayout(this);
	listview = new QListView(this);
	listview->addColumn(QString::fromUtf8("id"));
	listview->addColumn(QString::fromUtf8("表示"));
	listview->addColumn(QString::fromUtf8("zoom"));
	listview->addColumn(QString::fromUtf8("名前"));
	listview->setAllColumnsShowFocus(true);
	listview->setColumnAlignment(0, Qt::AlignHCenter);
	listview->setColumnAlignment(1, Qt::AlignHCenter);
	listview->setColumnAlignment(2, Qt::AlignHCenter);

	base_hbox->addWidget(listview);

	QVBox *vbox = new QVBox(this);
	QPushButton *add_button = new QPushButton(QString::fromUtf8("追加"), vbox);
	QPushButton *del_button = new QPushButton(QString::fromUtf8("削除"), vbox);
	QPushButton *mod_button = new QPushButton(QString::fromUtf8("変更"), vbox);
	new QWidget(vbox);
	base_hbox->addWidget(vbox);

	connect(add_button, SIGNAL(clicked()), this, SLOT(press_add()));
	connect(del_button, SIGNAL(clicked()), this, SLOT(press_del()));
	connect(mod_button, SIGNAL(clicked()), this, SLOT(press_mod()));

	connect(listview, SIGNAL(clicked(QListViewItem *, const QPoint &, int)),
			this, SLOT(click_row(QListViewItem *, const QPoint &, int)));

#ifndef QWS
	QPushButton *cancel = new QPushButton("Cancel", vbox);
	QPushButton *ok = new QPushButton("OK", vbox);
	ok->setDefault(true);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));
#endif

	db = new PoiDb(DB_POI_PATH);

	check[0].load(APP_PATH "/check-none.png");
	check[1].load(APP_PATH "/check.png");

	updateFileItem();
}

PoiDialog::~PoiDialog()
{
	delete db;
}

void PoiDialog::updateProgress(int now, int max)
{
	if (!progress)
		return;

	if (progress_timer.elapsed() > 100) {
		progress->setMax(max);
		progress->setStep(QString::fromUtf8("登録中・・・"), now);
		progress_timer.restart();
	}
}

void PoiDialog::importProgress(int now, int max, void *data)
{
	PoiDialog *myself = (PoiDialog *)data;

	myself->updateProgress(now, max);
}

void PoiDialog::press_add()
{
	PoiAddDialog dlg(PoiAddDialog::Add, this);

	if (dlg.exec() == QDialog::Accepted) {
		progress = new ProgressDialog(this);
		progress->show();
		progress->setCaption(QString::fromUtf8("POI 追加"));

		progress_timer.restart();

		db->importPoiCSV(dlg.getPoiFile(), dlg.getIconId(),
						 dlg.getZoom(), dlg.getName(), &importProgress, (void *)this);

		progress->hide();
		delete progress;
		progress = NULL;
	}
	updateFileItem();
}

void PoiDialog::press_del()
{
	QListViewItem *item = listview->currentItem();
	if (!item)
		return;

	if (QMessageBox::warning(this, QString::fromUtf8("削除の確認"),
							 QString::fromUtf8("%1 を削除してよろしいですか?").arg(item->text(3)),
							 QMessageBox::Cancel | QMessageBox::Default,
							 QMessageBox::Ok | QMessageBox::Escape) != QMessageBox::Ok) {
		return;
	}

	db->delFile(item->text(0).toInt());
	updateFileItem();
}

void PoiDialog::press_mod()
{
	QListViewItem *item = listview->currentItem();
	if (!item)
		return;

	int id = item->text(0).toInt();
//	printf("%d\n", id);

	PoiAddDialog dlg(PoiAddDialog::Mod, this);
	dlg.setId(id);
	if (dlg.exec() == QDialog::Accepted) {
//		printf("id:%d icon:%d name:%s\n", id, dlg.getIconId(), (const char *)dlg.getName().utf8());
		db->changePoiFile(id, dlg.getIconId(), dlg.getZoom(), dlg.getName());
	}
	updateFileItem();
}

void PoiDialog::updateFileItem()
{
	listview->clear();

	PoiDbFile *file = db->queryPoiFile(0);
	if (file) {

		if (!file->next()) {
			delete file;
			return;
		}

		do {
			QListViewItem *item = new QListViewItem(listview);
			item->setText(0, QString::number(file->getId()));
			item->setPixmap(1, check[file->getEnabled()]);
			item->setText(2, QString::number(file->getZoom()));
			item->setText(3, file->getName());

			item->setPixmap(3, QPixmap(db->getPoiIcon(file->getIconId())));
			listview->insertItem(item);
#if 0
			printf("%d %d %d %d %s\n",
				   file->getId(), file->getGrpId(), file->getIconId(),
				   file->getEnabled(),
				   (const char *)file->getName().utf8());
#endif
		} while(file->next());
		delete file;
	}
}

void PoiDialog::click_row(QListViewItem *item, const QPoint &point UNUSED, int column)
{
	if (!item || column != 1)
		return;

	int id = item->text(0).toInt();

	PoiDbFile *file = db->queryPoiFile(-1, id);

	if (!file)
		return;

	if (!file->next()) {
		delete file;
		return;
	}

	int enable = file->getEnabled();

	delete file;

	db->changePoiFileEnable(id, !enable);
	item->setPixmap(1, check[!enable]);
}
