/*
  qgmap.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __QGMAP_H__
#define __QGMAP_H__

#include "debug.h"

#define IMAGE_SIZE 256

#define MIN(a,b) ((a)>(b)?(b):(a))
#define MAX(a,b) ((a)>(b)?(a):(b))

#define KEY_SCROLL 10
#define SHIFT_MUL 10

#ifdef QWS
#define APP_PATH "/opt/QtPalmtop/share/qgmap/"
#define DB_POI_PATH "/home/zaurus/Settings/qgmap-poi.db"
#else
#define APP_PATH "./img/"
#define DB_POI_PATH "test.db"
#endif

#define NOIMG_FILE APP_PATH "/noimage.png"
#define LOADING_FILE APP_PATH "/loading.png"
#define MENUIMG_FILE APP_PATH "/menuicon.png"
#define SLIDERIMG_FILE APP_PATH "/slider.png"
#define NOBIMG_FILE APP_PATH "/nob.png"
#define MARK_FILE APP_PATH "/mark.png"
#define ICONS_FILE APP_PATH "/icons.png"

/* Default Settings */

#define DEFAULT_MAP_PATH "/mnt/card/gm_lite/imgs/"
#define DEFAULT_GAMMA "1.0"

#define DEFAULT_GPS_DEVICE "/dev/rfcomm0"
#define DEFAULT_PRE_COMMAND ""
#define DEFAULT_POST_COMMAND ""
#define DEFAULT_GPS_INTERVAL 1
#define DEFAULT_GPS_DATUM WGS84
#define DEFAULT_GPS_TIMEZONE +9

#define PI 3.14159265358979
#define EARTH_RA 6378137
#define EARTH_RB 6356752

#define SCALE_MAX_SIZE 100
#define SCALE_FONT_SIZE 16

#define CENTER_MARKER_SIZE 20

#define POI_MAX 500

#ifndef QWS
#define UNUSED __attribute__ ((unused))
#else
#define UNUSED
#endif

#define STR(x) _STR(x)
#define _STR(x) #x

#define MAXGPSSETTINGS 5
#define MAXMAPPATH 8

typedef enum datum {
	Tokyo,
	WGS84,
} Datum;

#define QCOPNAME "QPE/Application/qgmap"

class QApplication;
extern QApplication *pApp;
class MainWindow;
extern MainWindow *pMainWindow;

enum {
	EV_CUSTOM_REPAINT = 346798,

	/* 設定 */
	EV_SETTINGS_MENU,
	EV_SET_MAP_PATH,
	EV_GPS_MENU,

	/* 表示 */
	EV_DISP_MENU,
	EV_ZOOM_IN,
	EV_ZOOM_OUT,
	EV_TOGGLE_CENTER_MARKER,
	EV_TOGGLE_GRID,
	EV_SET_GAMMA,

	/* 表示->マーカー */
	EV_DISP_MARKER_MENU,
	EV_TOGGLE_MARKER,
	EV_TOGGLE_MARKER_0,
	EV_TOGGLE_MARKER_1,
	EV_TOGGLE_MARKER_2,
	EV_TOGGLE_MARKER_3,
	EV_TOGGLE_MARKER_4,
	EV_TOGGLE_MARKER_5,
	EV_TOGGLE_MARKER_6,
	EV_TOGGLE_MARKER_7,
	EV_TOGGLE_MARKER_8,
	EV_TOGGLE_MARKER_9,

	/* 位置 */
	EV_LOCATION_MENU,
	EV_GET_PHS_POS,
	EV_TOGGLE_GPS,
	EV_JUMP,
	EV_COPY_LOCATION,
	EV_PASTE_LOCATION,

	/* マーカージャンプ */
	EV_JUMP_MARKER_MENU,
	EV_JUMP_MARKER_0,
	EV_JUMP_MARKER_1,
	EV_JUMP_MARKER_2,
	EV_JUMP_MARKER_3,
	EV_JUMP_MARKER_4,
	EV_JUMP_MARKER_5,
	EV_JUMP_MARKER_6,
	EV_JUMP_MARKER_7,
	EV_JUMP_MARKER_8,
	EV_JUMP_MARKER_9,

	/* マーカー登録 */
	EV_SAVE_MARKER_MENU,
	EV_SAVE_MARKER_0,
	EV_SAVE_MARKER_1,
	EV_SAVE_MARKER_2,
	EV_SAVE_MARKER_3,
	EV_SAVE_MARKER_4,
	EV_SAVE_MARKER_5,
	EV_SAVE_MARKER_6,
	EV_SAVE_MARKER_7,
	EV_SAVE_MARKER_8,
	EV_SAVE_MARKER_9,

	/* POI */
	EV_POI_MENU,
	EV_POI_MANAGE,
	EV_POI_SEARCH,
};

#include <qstring.h>
#include <qobject.h>

static inline void *lookupObject(QObject *obj, const char *type, const QString &name)
{
	QObject *top = obj;

	while (top->parent())
		top = top->parent();

	return qt_find_obj_child(top, type, name.ascii());
}

#endif // __QGMAP_H__
